package ru.dbotthepony.mc.otm.entity

import net.minecraft.core.NonNullList
import net.minecraft.world.entity.EntityType
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.vehicle.AbstractMinecartContainer
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.mc.otm.block.CargoCrateBlock
import ru.dbotthepony.mc.otm.block.entity.CargoCrateBlockEntity
import ru.dbotthepony.mc.otm.menu.MinecartCargoCrateMenu
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry

class MinecartCargoCrate(
	type: EntityType<*>,
	val color: DyeColor?,
	level: Level,
	x: Double = 0.0,
	y: Double = 0.0,
	z: Double = 0.0
) : AbstractMinecartContainer(type, x, y, z, level) {
	val block = MRegistry.CARGO_CRATES.allBlocks[color] ?: throw NullPointerException("No cargo crate block for color $color!")
	val item = MItems.CARGO_CRATE_MINECARTS[color] ?: throw NullPointerException("No minecart cargo crate item for color $color!")

	private val itemStacksList = NonNullList.withSize(CargoCrateBlockEntity.CAPACITY, ItemStack.EMPTY)

	override fun getItemStacks(): NonNullList<ItemStack> {
		return itemStacksList
	}

	override fun clearItemStacks() {
		for (i in 0 until itemStacksList.size) {
			itemStacksList[i] = ItemStack.EMPTY
		}
	}

	override fun getDropItem(): Item {
		return item
	}

	override fun getMinecartType(): Type {
		return Type.CHEST
	}

	override fun getDefaultDisplayBlockState(): BlockState {
		return block.defaultBlockState().setValue(CargoCrateBlock.IS_OPEN, false)
	}

	override fun getDefaultDisplayOffset(): Int {
		return 8
	}

	override fun getContainerSize(): Int {
		return CargoCrateBlockEntity.CAPACITY
	}

	override fun createMenu(p_38222_: Int, p_38223_: Inventory): AbstractContainerMenu {
		return MinecartCargoCrateMenu(p_38222_, p_38223_, this)
	}

	override fun getPickResult(): ItemStack {
		return ItemStack(item, 1)
	}
}
